# Stack Detector Plugin for x64dbg

A powerful plugin for x64dbg that automatically detects, analyzes, and explains various stack-related issues in real-time during debugging. It provides detailed stack problem analysis with educational explanations targeted at both beginners and experienced users.

![Stack Detector Screenshot](screenshots/stack_detector_example.png)

## Features

- **Advanced Problem Detection**: Identifies common stack issues during debugging:
  - Stack Overflow
  - Buffer Overflow
  - Return Address Corruption
  - Stack Alignment Errors
  - Stack Frame Errors

- **Root Cause Analysis**: Attempts to determine the true source of problems, not just where they manifest
  - Detection of recursive calls
  - Identification of excessively large stack frames
  - Analysis of memory patterns around corrupted areas

- **Interactive Stack Trace**: Shows detailed stack trace with clickable addresses for easy navigation
  - Visual indicators for problematic frames
  - Highlighting of recursive calls
  - Identification of root cause locations

- **Educational Explanations**: Each issue includes detailed explanations about:
  - What the problem is and why it occurs
  - How to interpret the diagnostic information
  - Suggestions for fixing the issue

- **Automatic Analysis**: Can run automatically at breakpoints, steps, or exceptions

## Installation

1. Download the latest release from the [Releases](https://github.com/yourusername/stack-detector-plugin/releases) page
2. Copy the plugin DLL file (`StackDetector_x64.dp64` for 64-bit or `StackDetector_x32.dp32` for 32-bit) to your x64dbg plugins directory:
   - 64-bit: `{x64dbg installation folder}\release\x64\plugins\`
   - 32-bit: `{x64dbg installation folder}\release\x32\plugins\`
3. Restart x64dbg or use the plugin manager to load the plugin

## Usage

### Manual Analysis

1. Load your program in x64dbg and set breakpoints at areas of interest
2. When the program stops, use one of the following methods to analyze the stack:
   - Open the plugin menu and select "Scan Current Stack"
   - Use the keyboard shortcut (default: none, can be configured)
   - Type the command: `StackDetector.scan` in the command bar

### Automatic Analysis

To enable automatic stack analysis on each step or breakpoint:

1. Open the plugin menu and select "Enable Automatic Analysis"
2. Debug as usual, and the plugin will automatically scan the stack and report issues

### Understanding Results

When an issue is detected, the plugin displays a detailed report in the log window:

1. **Issue Header**: Shows the type of problem detected
2. **Issue Details**: Contains location, description, and register values
3. **Root Cause Analysis**: Identifies the probable source of the problem
4. **Educational Explanation**: Provides a beginner-friendly explanation of the issue
5. **Stack Trace**: Shows the call hierarchy leading to the problem, with special markers
6. **Additional Information**: Offers recommendations for fixing the issue

## Commands

The plugin adds the following commands to x64dbg:

- `StackDetector.scan`: Manually scan the current stack for issues
- `StackDetector.auto 0/1`: Disable/Enable automatic analysis
- `StackDetector.clear`: Clear all detected issues
- `StackDetector.about`: Display information about the plugin

## Building from Source

### Prerequisites

- Visual Studio 2019 or higher with C++17 support
- CMake 3.10 or higher

### Build Steps

1. Clone the repository:
   ```
   git clone https://github.com/yourusername/stack-detector-plugin.git
   cd stack-detector-plugin
   ```

2. Configure the project:
   ```
   mkdir build
   cd build
   cmake ..
   ```

3. Build the project:
   ```
   cmake --build . --config Release
   ```

4. The plugin will be generated in the `build/Release` directory

## Stack Analysis Features

The Stack Detector plugin uses several methods to identify stack issues:

1. **Stack Boundary Checks**: Analyzes if the stack pointer is within valid boundaries
2. **Repetition Pattern Search**: Identifies suspicious repeating patterns in memory
3. **Return Pointer Analysis**: Validates the return addresses of functions
4. **Recursion Detection**: Identifies excessive recursive calls
5. **Stack Alignment Analysis**: Checks if the stack is properly aligned according to platform requirements

## Acknowledgements

- [x64dbg](https://x64dbg.com) for the excellent debugging platform
- Contributors who helped test and improve the plugin

## License

This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.

## Contact

For bug reports, feature requests, or other inquiries, please create an issue on GitHub or contact the author at [your-email@example.com].
