# Плагин Stack Detector для x64dbg

Мощный плагин для x64dbg, который автоматически обнаруживает, анализирует и объясняет различные проблемы со стеком в реальном времени во время отладки. Он предоставляет подробный анализ проблем стека с обучающими пояснениями, ориентированными как на начинающих, так и на опытных пользователей.

![Скриншот Stack Detector](screenshots/stack_detector_example.png)

## Возможности

- **Продвинутое обнаружение проблем**: Выявляет распространенные проблемы стека во время отладки:
  - Переполнение стека (Stack Overflow)
  - Переполнение буфера (Buffer Overflow)
  - Повреждение адреса возврата (Return Address Corruption)
  - Ошибки выравнивания стека (Stack Alignment Errors)
  - Ошибки фрейма стека (Stack Frame Errors)

- **Анализ первопричин**: Пытается определить истинный источник проблемы, а не только место её проявления
  - Обнаружение рекурсивных вызовов
  - Идентификация чрезмерно больших фреймов стека
  - Анализ шаблонов памяти вокруг поврежденных областей

- **Интерактивная трассировка стека**: Показывает подробную трассировку стека с кликабельными адресами для удобной навигации
  - Визуальные индикаторы для проблемных фреймов
  - Подсветка рекурсивных вызовов
  - Идентификация первопричины проблемы

- **Обучающие пояснения**: Каждая проблема включает подробные объяснения о:
  - Сути проблемы и причинах её возникновения
  - Как интерпретировать диагностическую информацию
  - Предложения по исправлению проблемы

- **Автоматический анализ**: Может запускаться автоматически при точках останова, шагах или исключениях

## Установка

1. Загрузите последнюю версию со страницы [Releases](https://github.com/yourusername/stack-detector-plugin/releases)
2. Скопируйте файл DLL плагина (`StackDetector_x64.dp64` для 64-битной или `StackDetector_x32.dp32` для 32-битной версии) в директорию плагинов x64dbg:
   - 64-бит: `{папка установки x64dbg}\release\x64\plugins\`
   - 32-бит: `{папка установки x64dbg}\release\x32\plugins\`
3. Перезапустите x64dbg или используйте менеджер плагинов для загрузки плагина

## Использование

### Ручной анализ

1. Загрузите вашу программу в x64dbg и установите точки останова в интересующих областях
2. Когда программа останавливается, используйте один из следующих методов для анализа стека:
   - Откройте меню плагина и выберите "Сканировать текущий стек"
   - Используйте сочетание клавиш (по умолчанию: отсутствует, можно настроить)
   - Введите команду: `StackDetector.scan` в командной строке

### Автоматический анализ

Для включения автоматического анализа стека на каждом шаге или точке останова:

1. Откройте меню плагина и выберите "Включить автоматический анализ"
2. Отлаживайте как обычно, и плагин будет автоматически сканировать стек и сообщать о проблемах

### Понимание результатов

Когда обнаруживается проблема, плагин отображает подробный отчет в окне журнала:

1. **Заголовок проблемы**: Показывает тип обнаруженной проблемы
2. **Детали проблемы**: Содержит местоположение, описание и значения регистров
3. **Анализ первопричины**: Определяет вероятный источник проблемы
4. **Обучающее объяснение**: Предоставляет понятное для начинающих объяснение проблемы
5. **Трассировка стека**: Показывает иерархию вызовов, ведущую к проблеме, со специальными маркерами
6. **Дополнительные сведения**: Предлагает рекомендации по исправлению проблемы

## Команды

Плагин добавляет следующие команды в x64dbg:

- `StackDetector.scan`: Ручное сканирование текущего стека на наличие проблем
- `StackDetector.auto 0/1`: Отключение/включение автоматического анализа
- `StackDetector.clear`: Очистка всех обнаруженных проблем
- `StackDetector.about`: Отображение информации о плагине

## Сборка из исходного кода

### Предварительные требования

- Visual Studio 2019 или выше с поддержкой C++17
- CMake 3.10 или выше

### Шаги сборки

1. Клонируйте репозиторий:
   ```
   git clone https://github.com/yourusername/stack-detector-plugin.git
   cd stack-detector-plugin
   ```

2. Настройте проект:
   ```
   mkdir build
   cd build
   cmake ..
   ```

3. Соберите проект:
   ```
   cmake --build . --config Release
   ```

4. Плагин будет сгенерирован в директории `build/Release`

## Особенности анализа стека

Плагин Stack Detector использует несколько методов для выявления проблем со стеком:

1. **Проверка границ стека**: Анализирует, находится ли указатель стека в допустимых границах
2. **Поиск паттернов повторения**: Выявляет подозрительные повторяющиеся шаблоны в памяти
3. **Анализ указателей возврата**: Проверяет валидность адресов возврата функций
4. **Обнаружение рекурсии**: Выявляет избыточные рекурсивные вызовы
5. **Анализ выравнивания стека**: Проверяет корректность выравнивания стека согласно требованиям платформы

## Благодарности

- [x64dbg](https://x64dbg.com) за замечательную платформу для отладки
- Участникам, которые помогли тестировать и улучшать плагин

## Лицензия

Этот проект распространяется под лицензией MIT - см. файл [LICENSE](LICENSE) для получения подробной информации.

## Контакты

Для сообщений об ошибках, запросов функций или других запросов, пожалуйста, создайте issue на GitHub или свяжитесь с автором по адресу [your-email@example.com].
