#include <iostream>
#include <cstring>
#include <vector>
#include <functional>
#include <windows.h>
#include <intrin.h>

// Добавим глобальную переменную для проверки повреждения стека
volatile int canary = 0xDEADBEEF;

// Макросы для маркировки опасных мест
#ifdef _WIN64
    // Для x64 используем __nop intrinsic
    #define CORRUPTION_MARKER_START() do { \
        __nop(); __nop(); __nop(); __nop(); __nop(); \
    } while(0)
    #define CORRUPTION_MARKER_END() do { \
        __nop(); __nop(); __nop(); __nop(); __nop(); \
    } while(0)
#else
    // Для x86 можно использовать inline assembly
    #define CORRUPTION_MARKER_START() __asm { \
        nop \
        nop \
        nop \
        nop \
        nop \
    }
    #define CORRUPTION_MARKER_END() __asm { \
        nop \
        nop \
        nop \
        nop \
        nop \
    }
#endif

// Stack buffer overflow function
void stack_buffer_overflow() {
    std::cout << "Calling function with stack buffer overflow..." << std::endl;

    // Создаем локальные переменные в определенном порядке
    int local_canary = 0xCAFEBABE;
    char buffer[10];
    char large_string[256];

    std::cout << "Local canary before overflow: 0x" << std::hex << local_canary << std::endl;

    // Заполняем large_string повторяющимся паттерном
    for (int i = 0; i < 255; i++) {
        large_string[i] = 'A' + (i % 26);
    }
    large_string[255] = '\0';

    std::cout << "Copying 256 bytes into 10-byte buffer..." << std::endl;

    CORRUPTION_MARKER_START();
    strcpy(buffer, large_string);  // DON'T DO THIS IN REAL CODE!
    CORRUPTION_MARKER_END();

    std::cout << "Local canary after overflow: 0x" << std::hex << local_canary << std::endl;
}

// Function returning pointer to local variable
int* return_stack_pointer() {
    std::cout << "Returning pointer to local variable..." << std::endl;

    int local_var = 42;

    CORRUPTION_MARKER_START();
    return &local_var;  // DON'T DO THIS IN REAL CODE!
    CORRUPTION_MARKER_END();
}

// Function using uninitialized variable
void use_uninitialized() {
    std::cout << "Using uninitialized variables..." << std::endl;

    int uninitialized_array[4];
    int *ptr = &uninitialized_array[0];

    // Используем и модифицируем неинициализированные данные
    for(int i = 0; i < 4; i++) {
        CORRUPTION_MARKER_START();
        std::cout << "Value [" << i << "] = 0x" << std::hex << uninitialized_array[i] << std::endl;
        uninitialized_array[i] += 0xDEAD;
        CORRUPTION_MARKER_END();
        std::cout << "After modification [" << i << "] = 0x" << std::hex << uninitialized_array[i] << std::endl;
    }
}

// Recursive function without exit condition - will cause stack overflow
void infinite_recursion(int depth = 0) {
    CORRUPTION_MARKER_START();
    volatile char buffer[1024]; // Выделяем большой буфер на стеке

    // Заполняем буфер для гарантированного использования стека
    for(int i = 0; i < 1024; i++) {
        buffer[i] = (char)(i & 0xFF);
    }
    CORRUPTION_MARKER_END();

    if (depth % 100 == 0) {
        std::cout << "Recursion depth: " << std::dec << depth
                 << " (Stack pointer approx: 0x" << std::hex
                 << (size_t)&buffer << ")" << std::endl;
    }

    infinite_recursion(depth + 1);
}

// Function that modifies return address (Windows-specific version)
void corrupt_return_address() {
    std::cout << "Attempting to modify return address..." << std::endl;

    // Получаем указатель на адрес возврата
    void** return_addr_ptr;
#ifdef _WIN64
    // В x64 используем интринсик для получения адреса возврата
    return_addr_ptr = (void**)_AddressOfReturnAddress();
#else
    // В x86 используем inline assembly
    __asm {
        mov eax, [ebp+4]  // Получаем адрес возврата
        mov return_addr_ptr, eax
    }
#endif

    // Сохраняем оригинальный адрес возврата
    void* original_return = *return_addr_ptr;

    std::cout << "Return address before corruption: 0x"
              << std::hex << (size_t)original_return << std::endl;

    CORRUPTION_MARKER_START();
    // Модифицируем адрес возврата
    #ifdef _WIN64
        *return_addr_ptr = (void*)0xDEADBEEFDEADBEEFull;  // 64-bit значение для x64
    #else
        *return_addr_ptr = (void*)0xDEADBEEF;  // 32-bit значение для x86
    #endif
    CORRUPTION_MARKER_END();

    std::cout << "Return address after corruption: 0x"
              << std::hex << (size_t)*return_addr_ptr << std::endl;
}

// Function to run test with exception handling
void run_test(const std::function<void()>& test) {
    __try {
        test();
        std::cout << "Test completed (possibly without visible effects)" << std::endl;
    }
    __except(EXCEPTION_EXECUTE_HANDLER) {
        std::cout << "Caught Windows exception (code: 0x"
                 << std::hex << GetExceptionCode() << ")" << std::endl;
    }
}

int main() {
    // Disable Windows error dialog
    SetErrorMode(SEM_NOGPFAULTERRORBOX);

    std::cout << "Stack Corruption Demonstration Program (Windows version)" << std::endl;
    std::cout << "---------------------------------------------------" << std::endl;

    std::vector<std::pair<std::string, std::function<void()>>> tests = {
        {"Stack buffer overflow", stack_buffer_overflow},
        {"Return pointer to local variable", []() {
            int* ptr = return_stack_pointer();
            std::cout << "Value at returned pointer: " << *ptr << std::endl;
        }},
        {"Use uninitialized variable", use_uninitialized},
        {"Modify return address", corrupt_return_address},
        {"Infinite recursion (stack overflow)", []() { infinite_recursion(); }}
    };

    int choice;
    while (true) {
        std::cout << "\nSelect test to run:" << std::endl;
        for (size_t i = 0; i < tests.size(); i++) {
            std::cout << i + 1 << ". " << tests[i].first << std::endl;
        }
        std::cout << "0. Exit" << std::endl;
        std::cout << "Your choice: ";
        std::cin >> choice;

        if (choice == 0) {
            break;
        }

        if (choice > 0 && choice <= static_cast<int>(tests.size())) {
            run_test(tests[choice - 1].second);
        } else {
            std::cout << "Invalid choice" << std::endl;
        }
    }

    return 0;
}
